@@ -121,8 +121,6 @@ remove_lingering(struct pn_peer_link *link)
 void
 pn_peer_link_free(struct pn_peer_link *link)
 {
-    MsnSession *session;
-
     if (!link)
         return;
 
@@ -130,8 +128,6 @@ pn_peer_link_free(struct pn_peer_link *link)
     pn_info("link=%p", link);
 #endif
 
-    session = link->session;
-
     remove_lingering(link);
 
 #ifdef MSN_DIRECTCONN
