@@ -688,7 +688,7 @@ read_impl (PnNode *conn,
                 if (status == G_IO_STATUS_AGAIN)
                     return status;
 
-                if (str[0] == '\0')
+                if (!str || str[0] == '\0')
                     break;
 
 #ifdef PECAN_DEBUG_HTTP
@@ -969,7 +969,8 @@ foo_write (PnNode *conn,
 #endif
 
     http_conn->waiting_response = TRUE;
-    pn_timer_stop (http_conn->timer);
+    if (http_conn->timer)
+        pn_timer_stop (http_conn->timer);
 
     if (http_conn->cur)
         g_object_unref (http_conn->cur);
