@@ -462,7 +462,6 @@ show_send_to_mobile_cb(PurpleBlistNode *node, gpointer ignored)
 {
     PurpleBuddy *buddy;
     PurpleConnection *gc;
-    MsnSession *session;
     MsnMobileData *data;
 
     g_return_if_fail(PURPLE_BLIST_NODE_IS_BUDDY(node));
@@ -470,8 +469,6 @@ show_send_to_mobile_cb(PurpleBlistNode *node, gpointer ignored)
     buddy = (PurpleBuddy *) node;
     gc = purple_account_get_connection(buddy->account);
 
-    session = gc->proto_data;
-
     data = g_new0(MsnMobileData, 1);
     data->gc = gc;
     data->passport = buddy->name;
@@ -507,6 +504,18 @@ new_chat (MsnSession *session,
     return swboard;
 }
 
+static const char *get_my_alias(MsnSession *session)
+{
+    const char *alias;
+    PurpleAccount *account = msn_session_get_user_data(session);
+    alias = purple_account_get_alias(account);
+    if (!alias)
+        alias = purple_connection_get_display_name(account->gc);
+    if (!alias)
+        alias = msn_session_get_username(session);
+    return alias;
+}
+
 static void
 initiate_chat_cb(PurpleBlistNode *node, gpointer data)
 {
@@ -530,7 +539,7 @@ initiate_chat_cb(PurpleBlistNode *node, gpointer data)
     swboard->conv = serv_got_joined_chat(gc, swboard->chat_id, "MSN Chat");
 
     purple_conv_chat_add_user(PURPLE_CONV_CHAT(swboard->conv),
-                              msn_session_get_username(session), NULL, PURPLE_CBFLAGS_NONE, TRUE);
+                              get_my_alias(session), NULL, PURPLE_CBFLAGS_NONE, TRUE);
 }
 
 #if defined(PECAN_CVR)
@@ -730,9 +739,6 @@ tooltip_text (PurpleBuddy *buddy,
                 purple_notify_user_info_add_pair (user_info, _("Working"), user->media.title);
             }
         }
-
-        purple_notify_user_info_add_pair (user_info, _("Has you"),
-                                          ((user->list_op & (1 << MSN_LIST_RL)) ? _("Yes") : _("No")));
     }
 
     purple_notify_user_info_add_pair (user_info, _("Blocked"),
@@ -910,10 +916,16 @@ login (PurpleAccount *account)
 
     if (!purple_ssl_is_supported ())
     {
-        gc->wants_to_die = TRUE;
+#if PURPLE_VERSION_CHECK(2,3,0)
+        purple_connection_error_reason (gc,
+                PURPLE_CONNECTION_ERROR_NO_SSL_SUPPORT,
+                _("SSL support is needed for MSN. Please install a supported "
+                    "SSL library."));
+#else
         purple_connection_error (gc,
-                                 _("SSL support is needed for MSN. Please install a supported "
-                                   "SSL library."));
+                _("SSL support is needed for MSN. Please install a supported "
+                    "SSL library."));
+#endif
         return;
     }
 
@@ -1119,12 +1131,20 @@ send_im (PurpleConnection *gc,
     {
         struct pn_contact *contact;
         MsnSwitchBoard *swboard;
+        gboolean offline = FALSE;
+        struct pn_contact *user;
 
         contact = pn_contactlist_find_contact (session->contactlist, who);
-
         swboard = msn_session_find_swboard (session, who);
+        user = msn_session_get_contact (session);
 
         if (contact && contact->status == PN_STATUS_OFFLINE && !swboard)
+            offline = TRUE;
+
+        if (user->status == PN_STATUS_HIDDEN)
+            offline = TRUE;
+
+        if (offline)
         {
             pn_oim_session_request (session->oim_session,
                                     who,
@@ -1450,7 +1470,6 @@ rem_permit (PurpleConnection *gc,
 {
     MsnSession *session;
     struct pn_contact_list *contactlist;
-    struct pn_contact *user;
 
     session = gc->proto_data;
     contactlist = session->contactlist;
@@ -1461,12 +1480,8 @@ rem_permit (PurpleConnection *gc,
         g_return_if_reached ();
     }
 
-    user = pn_contactlist_find_contact (contactlist, who);
-
     pn_contactlist_rem_buddy (contactlist, who, MSN_LIST_AL, NULL);
-
-    if (user && user->list_op & MSN_LIST_RL_OP)
-        pn_contactlist_add_buddy (contactlist, who, MSN_LIST_BL, NULL);
+    pn_contactlist_add_buddy (contactlist, who, MSN_LIST_BL, NULL);
 }
 
 static void
@@ -1475,7 +1490,6 @@ rem_deny (PurpleConnection *gc,
 {
     MsnSession *session;
     struct pn_contact_list *contactlist;
-    struct pn_contact *user;
 
     session = gc->proto_data;
     contactlist = session->contactlist;
@@ -1486,12 +1500,8 @@ rem_deny (PurpleConnection *gc,
         g_return_if_reached ();
     }
 
-    user = pn_contactlist_find_contact (contactlist, who);
-
     pn_contactlist_rem_buddy (contactlist, who, MSN_LIST_BL, NULL);
-
-    if (user && user->list_op & MSN_LIST_RL_OP)
-        pn_contactlist_add_buddy (contactlist, who, MSN_LIST_AL, NULL);
+    pn_contactlist_add_buddy (contactlist, who, MSN_LIST_AL, NULL);
 }
 
 static void
@@ -1567,14 +1577,12 @@ chat_send (PurpleConnection *gc,
            const gchar *message,
            PurpleMessageFlags flags)
 {
-    PurpleAccount *account;
     MsnSession *session;
     MsnSwitchBoard *swboard;
     MsnMessage *msg;
     char *msgformat;
     char *msgtext;
 
-    account = purple_connection_get_account (gc);
     session = gc->proto_data;
     swboard = msn_session_find_swboard_with_id (session, id);
 
@@ -1940,6 +1948,8 @@ init_plugin (PurplePlugin *plugin)
 
 #ifndef STATIC_PECAN
 G_MODULE_EXPORT gboolean
+purple_init_plugin(PurplePlugin *plugin);
+G_MODULE_EXPORT gboolean
 purple_init_plugin(PurplePlugin *plugin)
 {
     plugin->info = &info;
@@ -1948,6 +1958,8 @@ purple_init_plugin(PurplePlugin *plugin)
 }
 #else
 gboolean
+purple_init_msn_pecan_plugin(void);
+gboolean
 purple_init_msn_pecan_plugin(void)
 {
     PurplePlugin *plugin = purple_plugin_new(TRUE, NULL);
