@@ -345,7 +345,7 @@ msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
 
             g_snprintf(tag, sizeof(tag),
                        "<FONT COLOR=\"#%02hx%02hx%02hx\">",
-                       colors[0], colors[1], colors[2]);
+                       (short) colors[0], (short) colors[1], (short) colors[2]);
 
             pre = g_string_append(pre, tag);
             post = g_string_prepend(post, "</FONT>");
@@ -912,10 +912,9 @@ pn_html_unescape (const char *url)
                 unsigned i;
                 for (i = 0; i < ARRAY_SIZE(refs); i++) {
                     struct ref ref = refs[i];
-                    int len = strlen(ref.name);
+                    int len = MIN(strlen(ref.name), (size_t)(end - src));
                     if (strncmp (src, ref.name, len) == 0) {
                         dest = g_string_append (dest, ref.value);
-                        src += len;
                         break;
                     }
                 }
